@@ -9,9 +9,9 @@
  * 
  * JasPer License Version 2.0
  * 
+ * Copyright (c) 2001-2006 Michael David Adams
  * Copyright (c) 1999-2000 Image Power, Inc.
  * Copyright (c) 1999-2000 The University of British Columbia
- * Copyright (c) 2001-2003 Michael David Adams
  * 
  * All rights reserved.
  * 
@@ -306,6 +306,7 @@ static jas_image_cmpt_t *jas_image_cmpt_create(uint_fast32_t tlx, uint_fast32_t
 		return 0;
 	}
 
+	cmpt->type_ = JAS_IMAGE_CT_UNKNOWN;
 	cmpt->tlx_ = tlx;
 	cmpt->tly_ = tly;
 	cmpt->hstep_ = hstep;
@@ -425,6 +426,10 @@ int jas_image_readcmpt(jas_image_t *image, int cmptno, jas_image_coord_t x,
 		return -1;
 	}
 
+	if (!jas_matrix_numrows(data) || !jas_matrix_numcols(data)) {
+		return -1;
+	}
+
 	if (jas_matrix_numrows(data) != height || jas_matrix_numcols(data) != width) {
 		if (jas_matrix_resize(data, height, width)) {
 			return -1;
@@ -478,6 +483,10 @@ int jas_image_writecmpt(jas_image_t *image, int cmptno, jas_image_coord_t x, jas
 		return -1;
 	}
 
+	if (!jas_matrix_numrows(data) || !jas_matrix_numcols(data)) {
+		return -1;
+	}
+
 	if (jas_matrix_numrows(data) != height || jas_matrix_numcols(data) != width) {
 		return -1;
 	}
@@ -1298,7 +1307,7 @@ jas_image_t *jas_image_chclrspc(jas_image_t *image, jas_cmprof_t *outprof,
 	jas_cmcmptfmt_t *outcmptfmts;
 
 #if 0
-fprintf(stderr, "IMAGE\n");
+jas_eprintf("IMAGE\n");
 jas_image_dump(image, stderr);
 #endif
 
@@ -1433,9 +1442,9 @@ jas_image_dump(image, stderr);
 	jas_image_destroy(inimage);
 
 #if 0
-fprintf(stderr, "INIMAGE\n");
+jas_eprintf("INIMAGE\n");
 jas_image_dump(inimage, stderr);
-fprintf(stderr, "OUTIMAGE\n");
+jas_eprintf("OUTIMAGE\n");
 jas_image_dump(outimage, stderr);
 #endif
 	return outimage;
